#ifndef ACKTABLE_HH
#define ACKTABLE_HH


#include <click/element.hh>
#include <click/etheraddress.hh>
#include <click/ipaddress.hh>
#include <click/timer.hh>
#include <click/hashmap.hh>
#include <click/click_adhoc.hh>
#include <click/list.h>
#include <click/click_dsr.hh>

class ACKEntry
{

private:

	/* Data Members */
	int _ack_id;
        struct timeval _time_added;
        Packet *_p;

public:
	/* Constructors */

	ACKEntry();
	ACKEntry(int,struct timeval,Packet *p);
	
	/* Access Functions */

	int ack_id() const{ return _ack_id; }
	struct timeval time_added() const{ return _time_added; }
        Packet *p() { return _p;}

	/* Other functions */

	LIST_ENTRY(ACKEntry) ln_ackentry;
};

LIST_HEAD(ACKentry_head, ACKEntry);


class ACKTable : public Element { 

private:
  
  Timer _expire_timer;
  int _hello_interval_jiffies;

	/* Private function */
  	void probe_acktable();

public:
  
 	ACKTable();
 	~ACKTable();

	ACKentry_head acache;
 
 	const char *class_name() const		{ return "ACKTable"; }
 	const char *processing() const		{ return PUSH; }
 
 	ACKTable *clone() const;
 	int configure(const Vector<String> &, ErrorHandler *);
 	void notify_ninputs(int n);

 	int initialize(ErrorHandler *);
 	void uninitialize();

 	void push(int, Packet *);
  	ACKEntry* findACKEntry(int );
  	void addInfoToACKTable(Packet *);
	static void expire_hook(Timer *, void *);

 
 	static const int TIMER_INTERVAL = 50;
};
#endif





















