/*
 * output.{cc,hh} -- element outputs packet on corresponding interface(s)
 * Audun Tornquist
 */
#include <click/config.h>
#include <click/package.hh>
#include <click/click_ether.h>
#include <click/etheraddress.hh>
#include <click/ipaddress.hh>
#include <click/confparse.hh>
#include <click/bitvector.hh>
#include <click/error.hh>
#include <click/glue.hh>
#include <click/click_dsr.hh>
#include <click/click_adhoc.hh>
#include <click/click_ip.h>
#include <click/router.hh>
#include <click/standard/scheduleinfo.hh>

#include "arptable.hh"
#include "wireless.h"
#include "iwlib.h"

#include <sys/ioctl.h>
#include <net/ethernet.h>
#include <linux/if_ether.h>
#include <stdlib.h>


ArpTable::ArpTable()
{
	_me = new IPAddress;
        _me_ether=new EtherAddress;
	_ipaddress = new IPAddress[50];
	_ethaddress = new EtherAddress[50];
	_count = 0;
  	MOD_INC_USE_COUNT;
	add_input();
	add_input();
	add_output();
	add_output();
}

ArpTable::~ArpTable()
{
  MOD_DEC_USE_COUNT;
}

void
ArpTable::notify_noutputs(int n)
{
  // allow for one output for each connected interface
  set_noutputs(n);
}

ArpTable *
ArpTable::clone() const
{
  return new ArpTable();
}

int
ArpTable::configure(const Vector<String> &conf, ErrorHandler *errh)
{
	if (cp_va_parse(conf, this, errh, cpKeywords,
		        "IPADDRESS", cpIPAddress, "ip address", _me,
                        "ETHADDRESS",cpEthernetAddress,"ethernet address",_me_ether,
			0)<0)
		return -1;
          addEntry(*_me,*_me_ether);
	return 0;
}
int
ArpTable::initialize(ErrorHandler *errh)
{
  return 0;
}

void
ArpTable::push(int port, Packet *p_in)
{

  	EtherAddress src_ether;
	char power[30];

	if ( port == 0)
	{
  	         unsigned dst_ip = p_in->dst_ip_anno();
  		/*  add ethernet header */
  		if (WritablePacket *q = p_in->push(sizeof(click_ether))) 
		{
    			click_ether *ether = 
				reinterpret_cast<click_ether *>(q->data());
    			/*  todo: have this as argument */
    			ether->ether_type = htons(0x0800);
	
	
			EtherAddress dst_ether,src_ether;
			IPAddress dst_addr(dst_ip);

			dst_ether = ConvertIPtoMAC(dst_addr);
			src_ether = ConvertIPtoMAC(*_me);

			memcpy(&ether->ether_shost,src_ether.data(),6);
			memcpy(&ether->ether_dhost,dst_ether.data(),6);

	  		output(0).push(q);
		}
		else
			click_chatter("*output: could not get ethernet header\n");
	} /* End of port == 0 */
	if (port==1)
	  {
           const click_ip *ip = (const click_ip*)(p_in->data() + sizeof(click_ether) );
           if(ip->ip_p!=IP_PROTO_DSR)
              {
               output(1).push(p_in);
               return;
              }
	    /* Get the ethernet address from the packet*/

	    EtherAddress mac = extractEthernetAddress(p_in);
	    int result = findEntry(mac);
	    if (!result)
	      {
		IPAddress tap_ip = extractIPAddress(p_in);
                click_chatter("IP: %s",tap_ip.s().cc());
		addEntry(tap_ip,mac);
	      }
	    output(1).push(p_in);
	  }
}

IPAddress ArpTable::extractIPAddress(Packet *p)
{
  const click_ip *ip = (const click_ip*)(p->data() + sizeof(click_ether) );
  const click_dsr *dsr = (const click_dsr*)(p->data() + sizeof(click_ether) + sizeof(click_ip));
 int payload=ntohs(dsr->dsr_len);
  const click_dsr_option *dsr_option = (const click_dsr_option *)(\
			p->data() + sizeof(click_ether) + sizeof(click_ip) + sizeof(click_dsr));

  if (dsr_option->dsr_type == DSR_TYPE_RREQ)
    {
      	const click_dsr_rreq * dsr_rreq = (const click_dsr_rreq *)(p->data()+\
					sizeof(click_ip)+sizeof(click_ether)+sizeof(click_dsr));
	//click_dsr_rreq *dsr_rreq = (click_dsr_rreq*)dsr_option;
      int hop_count = (dsr_rreq->dsr_len - 6)/4;
      int dsr_len = dsr_rreq->dsr_len;
      if (hop_count == 0)
	{
	  IPAddress src_addr(ip->ip_src.s_addr);
	  return (src_addr);
	}
      else
	{
	  const click_dsr_addr *dsr_addr = (const click_dsr_addr*)(p->data() + sizeof(click_ip) + sizeof(click_ether) +\
								       sizeof(click_dsr) + sizeof(click_dsr_rreq));	
	  int i=0;
	  while (i!=hop_count-1)
	    {
	    dsr_addr++;
	    i++;
	    }
	  IPAddress last_hop(dsr_addr->dsr_dst.s_addr);
	  return (last_hop);
	}
    }/* End of DSR_RREQ option */
  if (dsr_option->dsr_type == DSR_TYPE_RREP)
    {
	const click_dsr_rrep *dsr_rrep= (const click_dsr_rrep*)dsr_option;
      int hop_count=(dsr_rrep->dsr_len -1)/4;
  	const click_dsr_addr * dsr_addr=(const click_dsr_addr *)(p->data()+sizeof(click_ether)+\
					sizeof(click_ip)+sizeof(click_dsr)+\
					sizeof(click_dsr_rrep));
      if (hop_count == 2)
	{
	  IPAddress src_addr(ip->ip_src.s_addr);
	  return (src_addr);
	}
      else /* Hop count !=0 */
	{
 	   const click_dsr_source *dsr_source;
        	click_dsr_addr *dsr_source_addr;
       
	/* Check to see if the power header is present */
          if(payload>(sizeof(click_dsr_rrep)+hop_count*sizeof(click_dsr_addr)))
          {
           
 	    const click_dsr_option *dsr_option_new=(const click_dsr_option*)(p->data()+\
sizeof(click_ip)+ sizeof(click_ether) + \
				     sizeof(click_dsr)+\
				     sizeof(click_dsr_rrep)+\
				     hop_count*sizeof(click_dsr_addr));

           //Get length of unknown optional header
           int offset=sizeof(click_dsr_option)+dsr_option_new->dsr_len;

 	   dsr_source=(click_dsr_source*)(p->data()+\
							 sizeof(click_ip)+ sizeof(click_ether) + \
				     sizeof(click_dsr)+\
				     sizeof(click_dsr_rrep)+\
				     hop_count*sizeof(click_dsr_addr)+offset);

 	      dsr_source_addr=(click_dsr_addr*)(p->data()+\
					sizeof(click_ip)+\
				 	sizeof(click_ether)+\			  
					sizeof(click_dsr)+\
					sizeof(click_dsr_rrep)+\
				hop_count*sizeof(click_dsr_addr)+offset+\
					sizeof(click_dsr_source));
	 }

        else
        {
 	   dsr_source=(click_dsr_source*)(p->data()+\
							 sizeof(click_ip)+ sizeof(click_ether) + \
				     sizeof(click_dsr)+\
				     sizeof(click_dsr_rrep)+\
				     hop_count*sizeof(click_dsr_addr));

 	      dsr_source_addr=(click_dsr_addr*)(p->data()+\
					sizeof(click_ip)+\
				 	sizeof(click_ether)+\		  
					sizeof(click_dsr)+\
					sizeof(click_dsr_rrep)+\
				hop_count*sizeof(click_dsr_addr)+\
					sizeof(click_dsr_source));

       }
	 int source_hops= (dsr_source->dsr_len-2)/4;
	 int segments=ntohs(dsr_source->dsr_flags);
	 int index=source_hops-segments;
         if (index == 0)
	{
	  IPAddress src_addr(ip->ip_src.s_addr);
	  return (src_addr);
	}

         int i=0;
      	 while(i!=index)
   	 {
	   dsr_source_addr++;
           i++;
   	 }
	 IPAddress last_hop(dsr_source_addr->dsr_dst.s_addr);
	 return (last_hop);
	}
  }/* End of RREP */
  
  /* SOURCE ROUTED PACKET */
  if (dsr_option->dsr_type == DSR_TYPE_SOURCE_ROUTE) 
    {
 	  const click_dsr_source *dsr_source=(click_dsr_source*)(p->data()+\
							 sizeof(click_ip)+ sizeof(click_ether) + \
								  sizeof(click_dsr));
            const click_dsr_addr *dsr_addr=(const click_dsr_addr*)(p->data()+\
	    sizeof(click_ip)+sizeof(click_ether)+sizeof(click_dsr)+sizeof(click_dsr_source));

          unsigned hop_count = (dsr_source->dsr_len - 2) / 4;
	  unsigned segments = ntohs(dsr_source->dsr_flags);
	  unsigned prev_hop = hop_count - segments;
          int i=0;
         if (prev_hop == 0)
	{
	  IPAddress src_addr(ip->ip_src.s_addr);
	  return (src_addr);
	}
      	  while(i!=prev_hop)
   	  {
	    dsr_addr++;
            i++;
   	  }
	  IPAddress last_hop(dsr_addr->dsr_dst.s_addr);
	  return (last_hop);
    }/* End of Source Routed Packet */
 if (dsr_option->dsr_type == DSR_TYPE_ACK) 
   {
	  click_dsr_ack *dsr_ack=(click_dsr_ack*)(p->data()+\
			sizeof(click_ip)+sizeof(click_ether)+sizeof(click_dsr));
	  IPAddress ack_src(dsr_ack->dsr_src.s_addr);
	  return (ack_src);
   } /* End of ACK packet */
 if (dsr_option->dsr_type == DSR_TYPE_RERR) 
   {
	click_dsr_source *dsr_source = (click_dsr_source *)( p->data() + \
				   sizeof(click_ip) + sizeof(click_dsr) + \
				   sizeof(click_dsr_rerr) + sizeof(click_ether)+\
				   sizeof(click_dsr_addr));

	click_dsr_addr *dsr_source_addr = (click_dsr_addr *)( p->data() + \
				   sizeof(click_ip) + sizeof(click_ether) + sizeof(click_dsr) + \
				   sizeof(click_dsr_rerr) + \
				   sizeof(click_dsr_addr) +\
				   sizeof(click_dsr_source));

	int source_hops= (dsr_source->dsr_len-2)/4;
	int segments=ntohs(dsr_source->dsr_flags);
	int index=source_hops-segments;
	int i=0;
	while(i!=index)
	  {
	    dsr_source_addr++;
	    i++;
	  }
	  IPAddress last_hop(dsr_source_addr->dsr_dst.s_addr);
	  return (last_hop);
   }/* ENd of Route Error Packet */
} /* End of extractIPAddress */

EtherAddress ArpTable::extractEthernetAddress(Packet *p)
{
   const click_ether *e = (const click_ether *)p->data();
	return EtherAddress(e->ether_shost);
}

void ArpTable::addEntry(IPAddress ip, EtherAddress eth)
{
  _count++;
  _ipaddress[_count-1] = ip;
  _ethaddress[_count-1] = eth;
}

int ArpTable::findEntry(EtherAddress eth)
{

	int i;
        for (i = 0; i <_count; i++)
	{
		if ( getmacaddress(i)==eth)
                {
		  return 1; /* MAC Address exists */
                }
	}
        return 0;
}

EtherAddress ArpTable::getmacaddress(int i)
{
  return _ethaddress[i];
}

IPAddress ArpTable::getipaddress(int i)
{
  return _ipaddress[i];
}


EtherAddress ArpTable::ConvertIPtoMAC(IPAddress ip)
{

	int i;
	
	for (i=0;i<_count;i++)
	{
		if ( getipaddress(i)==ip )
		{
		    return getmacaddress(i);
		}
	}

}

char ArpTable::get_rssi(EtherAddress mac)
{
	int dbm;
	int skfd;
	int i=0;	
	char mac_addr[18];
	char power[30];
	sprintf(mac_addr,"%s",mac.s().cc());

	skfd = socket(AF_INET,SOCK_DGRAM,0);
	if (skfd < 0)
		click_chatter("Error opening the socket\n");
	
	char *ifname="eth1";
	cout << "** MAC ADDRESS "<< mac_addr<< endl;

	char rssi = compute_rssi(skfd,ifname,(char *)mac.s().cc());

	//dbm = rssi;
	
	return rssi;
}

char ArpTable::compute_rssi(int skfd, char *ifname, char* mac_addr)
{
	struct iwreq wrq;
	char buffer[(sizeof(struct iw_quality) + sizeof(struct sockaddr)) * \
			IW_MAX_SPY];	
	struct sockaddr hwa[IW_MAX_SPY];
	struct iw_quality qual[IW_MAX_SPY];

	int has_range = 0;
	int n,i,j;
         char RSSI = 0;

	for (j=0;j<5;j++)
	  { 
		strncpy(wrq.ifr_name,ifname, IFNAMSIZ);
		wrq.u.data.pointer = (caddr_t)buffer;
		wrq.u.data.length =  IW_MAX_SPY;
		wrq.u.data.flags = 0;

		if (ioctl(skfd,SIOCGIWSPY,&wrq) < 0) 
		{
			click_chatter("%s Interface doesn't support wireless \
					statistic collection \n",mac_addr);
		}

		n = wrq.u.data.length;
		has_range = 1;
		memcpy(hwa,buffer,n*sizeof(struct sockaddr));
		memcpy(qual,buffer+n*sizeof(struct sockaddr),\
			n*sizeof(struct iw_quality));

		for (i=0;i<n;i++)
		{
			if (qual[i].updated & 0x7)
			{
                                cout<<"Am coming in updated"<<endl;
				RSSI = qual[i].level;
				char *x;
				x = pr_ether((unsigned char*)hwa[i].sa_data);
				if (strcmp(x,mac_addr) ==0)
					RSSI = qual[i].level;
			}
			else
				RSSI = qual[i].level;
		}
		if ( RSSI == 0 )
		{
			RSSI = 0;
		}
	  }

	//return (RSSI-256);
	return (RSSI);
}

char *
ArpTable::pr_ether(unsigned char *ptr)
{
	static char buffer[64];
  sprintf(buffer, "%02X:%02X:%02X:%02X:%02X:%02X",
	  (ptr[0] & 0xFF), (ptr[1] & 0xFF), (ptr[2] & 0xFF),
	  (ptr[3] & 0xFF), (ptr[4] & 0xFF), (ptr[5] & 0xFF)
  );
  return(buffer);
}

	
EXPORT_ELEMENT(ArpTable)
