#ifndef ARPTABLE_HH
#define ARPTABLE_HH
/*
 * =c
 * ArpTable
 * =s duplication
 * sends packet to the recorded interfaces for the destination route
 * =d
 * ArpTable broadcast packets on all configured interfaces or
 * selects the correct interface for unicast packets.
 *
 * =e
 * Intended to be used in conjunction with Paint and DSRARPCache.
 *
 * dsr_out :: ArpTable;
 * // from output
 * dsr_out[0] -> Print(out_eth0) -> Queue -> ToDevice(eth0);
 * dsr_out[1] -> Print(out_eth1) -> Queue -> ToDevice(eth1);
 *
 * =a Paint, DSRARPCache
 */
#include <click/element.hh>
#include "routetable.hh"
#define NUM_PKT_ENTRIES 5


class ArpTable : public Element 
{      
	IPAddress *_me;
	EtherAddress *_me_ether;
        IPAddress *_ipaddress;
        EtherAddress *_ethaddress;
        int _count;

 public:
  
  ArpTable();
  ~ArpTable();
  
  const char *class_name() const	{ return "ArpTable"; }
  const char *processing() const	{ return AGNOSTIC; }
  void notify_noutputs(int);
  
  ArpTable *clone() const;
  int configure(const Vector<String> &, ErrorHandler *);
  int initialize(ErrorHandler *errh); 
  void push(int port, Packet *);
  EtherAddress getmacaddress(int i);
  IPAddress getipaddress(int i);
  EtherAddress ConvertIPtoMAC(IPAddress ip);
  void addEntry(IPAddress ip, EtherAddress eth);
  int findEntry(EtherAddress eth);
  EtherAddress extractEthernetAddress(Packet *p);
  IPAddress extractIPAddress(Packet *p);
  char *pr_ether(unsigned char *ptr);
  char compute_rssi(int skfd, char *ifname, char *mac_addr);
  char get_rssi(EtherAddress mac);
};
#endif





































