/*
 * getetheraddress.{cc,hh} -- element stores ethernet source addres found in 
 * ethernet header in user annotation 1-7
 * Audun Tornquist
 */

#include <click/config.h>
#include <click/package.hh>
#include "getetheraddress.hh"
#include "routingtable.hh"
#include <click/ipaddress.hh>
#include <click/confparse.hh>
#include <click/error.hh>
#include <click/glue.hh>
#include <click/click_ether.h>
#include <click/standard/alignmentinfo.hh>
#include <click/packet_anno.hh>

#ifdef CLICK_SIM
using namespace CLICKSIM;
#endif

GetEtherAddress::GetEtherAddress()
  : Element(1, 1)
{
  MOD_INC_USE_COUNT;
}

GetEtherAddress::~GetEtherAddress()
{
  MOD_DEC_USE_COUNT;
}

GetEtherAddress *
GetEtherAddress::clone() const
{
  return new GetEtherAddress;
}

int
GetEtherAddress::initialize(ErrorHandler *)
{
  return 0;
}

Packet *
GetEtherAddress::simple_action(Packet *p_in)
{
  WritablePacket *p = static_cast<WritablePacket *>(p_in);
  click_ether *ether = reinterpret_cast<click_ether *>(p->data());  
  // copy ethernet address
  for (int i = ADHOC_ETHERNET_ANNO; i < ADHOC_ETHERNET_ANNO + 6; i++)
    p->set_user_anno_c(i,ether->ether_shost[i-ADHOC_ETHERNET_ANNO]);

  // all done!
  return p;
}

EXPORT_ELEMENT(GetEtherAddress)
