#ifndef REQUESTTABLE_HH
#define REQUESTTABLE_HH

/*
 * =c
 * AODHello(SADDR [, INTERVAL])
 * =s AODV, route maintanance
 * Send out route replies to announce ourselves to the neighborhood
 * =d
 * So that the protocol does not have to rely on link layer connectivity
 * information, the router can be configured to send out route replies
 * to announce our presence to our neighbors every INTERVAL ms.  SADDR
 * is the source address of our interface.
 *
 * =a
 * AODVRouteReply
 */

#include <click/element.hh>
#include <click/etheraddress.hh>
#include <click/ipaddress.hh>
#include <click/timer.hh>
#include <click/hashmap.hh>
#include <click/click_adhoc.hh>
#include <click/list.h>
#include <click/click_dsr.hh>

class RequestEntry
{

private:

	/* Data Members */
	IPAddress _srcIp;
	IPAddress _dstIp;
	int _ttl;
	struct timeval _time_last_orig;
	int _num_rt_disc;
	double _time_interval;//in millisecs
	int _request_id;	
  
public:
	/* Constructors */

	RequestEntry();
	RequestEntry(IPAddress,IPAddress,int,struct timeval,int,double,int);
	
	/* Access Functions */

	IPAddress src_IP() const{ return _srcIp; }
	IPAddress dst_IP() const{ return _dstIp; }
	int ttl() const{ return _ttl; }
	struct timeval time_last_orig() const{ return _time_last_orig; }
	int num_rt_disc() const{ return _num_rt_disc; }
	void set_num_rt_disc(int n) { _num_rt_disc=n; }
	double time_interval() const{ return _time_interval; }
	void set_time_interval(double time_int){ _time_interval=time_int; }
	int request_id() const{ return _request_id; }

	/* Other functions */

	LIST_ENTRY(RequestEntry) ln_reqentry;
	

};

LIST_HEAD(Reqentry_head, RequestEntry);


class RequestTable : public Element { 

private:
  
  Timer _expire_timer;
  int _hello_interval_jiffies;

	/* Private function */
  	void expire_hello();

public:
  
 	RequestTable();
 	~RequestTable();

	Reqentry_head rcache;
        int _request_id;
	IPAddress *me;
 
 	const char *class_name() const		{ return "RequestTable"; }
 	const char *processing() const		{ return PUSH; }
 
 	RequestTable *clone() const;
 	int configure(const Vector<String> &, ErrorHandler *);
 	void notify_ninputs(int n);

 	int initialize(ErrorHandler *);
 	void uninitialize();

 	void push(int, Packet *);



        RequestEntry* findRequestEntry(IPAddress dst, IPAddress src);
        RequestEntry* findRequestEntry(int, IPAddress);
        //Look up the requesttable for a particular entry

        void addInfoToReqTable(IPAddress srcIp, IPAddress dstIp, int request_id);
        //Add an  entry to the request table







        Packet * send_query_for(unsigned src, unsigned dst, int request_id, unsigned char ttl); //Send out a route request packet for a particular destination node


	Packet * propagate_query(Packet *p);
       //Propagate an incoming query for an unknown destination after adding the current node in the route request address list

        static void expire_hook(Timer *, void *);

 
 	static const int TIMER_HELLO_INTERVAL = 1000;
};
#endif





















