#ifndef ROUTETABLE_HH
#define ROUTETABLE_HH

#include <click/element.hh>
#include <click/etheraddress.hh>
#include <click/ipaddress.hh>
#include <click/timer.hh>
#include <click/hashmap.hh>
#include <click/click_adhoc.hh>
#include <assert.h>
#include <click/maptable.hh>


//Class represents the SendBufferEntry structure used to buffer packets with 
//unknown destination
class SendBufferEntry
{
   private:
    //Packet pointer 
    Packet * _p;
    //Time of addition of packet
    struct timeval _time_added;

   public:
  
   /* Constructor*/

  SendBufferEntry()
  {
    
  }

  //This constructor creates a sendbuffer entry
  SendBufferEntry(Packet * p_in, struct timeval time_added)
 {
   _p=p_in;
   _time_added.tv_sec=time_added.tv_sec;
   _time_added.tv_usec=time_added.tv_usec;
 }

  

  /* Access functions*/
  Packet * p() { return _p; }
  struct timeval time_added(){ return _time_added;}

  /* Other functions*/

 //Create a list entry
  LIST_ENTRY(SendBufferEntry) ln_sendbuffentry;

};

//Create  the  SendBuffer list head
LIST_HEAD(SendBufferEntry_head, SendBufferEntry);


//This is the SendBuffer class with the access functions
class SendBuffer
{
  public :
   SendBufferEntry_head sendbuffcache;
   //Constructor
   SendBuffer()
   {
     LIST_INIT(&sendbuffcache);
   }
  
  //Function to add a packet to the sendbuffer
  void AddEntry(Packet *p)
 {
  	 struct timeval tv;
  	 click_gettimeofday(&tv);
   	 SendBufferEntry *s,*sb;
   	 s=new SendBufferEntry(p,tv);
  	 if(sendbuffcache.lh_first==NULL)
  	 {
   	   LIST_INSERT_HEAD(&sendbuffcache,s,ln_sendbuffentry);
 	 }
 	 else
  	{
   		for(sb=sendbuffcache.lh_first;sb->ln_sendbuffentry.le_next!=NULL;sb=sb->ln_sendbuffentry.le_next);
  
   	LIST_INSERT_AFTER(sb,s,ln_sendbuffentry);
  	}
 }

};

   
//This is the Routetable class derived from Element
class RouteTable : public Element { 


  unsigned _tailroom; 
  //Stores the current node IP Address
  IPAddress *me;


private:

        //Timer for the element
  	Timer _expire_timer;
        //Task handler
  	Task *t;
  	int _hello_interval_jiffies;
        //Acknowledgement ID for ACK packets
	int _ack_id;
  
  void probe_sendbuffer();//Function to probe the sendbuffer 

  Packet* addDSRHeader(Packet *, PathIPAddress);//Add a Fixed Header to the incoming packet

  Packet* StripHeaders(Packet *);//Strip all the DSR headers in the incoming packet

  PathIPAddress createReplyRoute(Packet *);// create a reply route from the RReq

  PathIPAddress extractReplyRoute(Packet *);// extract a reply route from the RRep

  PathIPAddress extractSourceRoute(Packet *);// extract a source route from pkt

  PathIPAddress reverse_route(PathIPAddress p_ip);//reverse route 

  Packet* create_dsr_rrep(IPAddress src, IPAddress dst, PathIPAddress \
			  reply_route, PathIPAddress source_route, int id );
	// Create the Route reply packet with the entire source route
  

Packet* create_dsr_rerr(IPAddress , IPAddress ,IPAddress, PathIPAddress);
	// Create the Route error packet with the entire source route


  Packet* create_dsr_ack(unsigned, unsigned, unsigned short);
	// Create the  ACK packet

  Packet* forwardPacket(Packet *);
	// Forward the Reply packet to the next hop

  Packet* forwardDataPacket(Packet *);
	// Forward the Data  packet to the next hop

 Packet* forwardError(Packet *);
	// Forward the Route Error  packet to the next hop

bool findMeInRequest(Packet *);
	// Checks if the current node is a part of the route in a Route Request packet 

public:
  //Pointer to a LinkCache object that stores the routes
  LinkCache *MyLinkCache; 

  //Pointer to a Maptable object that maps Node Id to its IP Address
  MapTable *MyMapTable;

  //Pointer to a SendBuffer Object
  SendBuffer *MySendBuffer;

  RouteTable();
  ~RouteTable();
  
  //Click specific functions`
  const char *class_name() const		{ return "RouteTable"; }
  const char *processing() const		{ return PUSH; }
  
  RouteTable *clone() const;
  int configure(const Vector<String> &, ErrorHandler *);
  void notify_ninputs(int n);

  //Called by the task expiry handler`
  void run_scheduled();

  int initialize(ErrorHandler *);
  void uninitialize();

  void push(int, Packet *);
  static void expire_hook(Timer *, void *);


  static const int TIMER_INTERVAL = 1000;// Timer for probing sendbuffer
};



#endif






