#ifndef CLICK_ADHOC_H
#define CLICK_ADHOC_H

#include <click/hashmap.hh>
#include <click/sync.hh>
#include <click/llrpc.h>
#include <click/vector.cc>

#ifdef CLICK_SIM
namespace CLICKSIM {
#endif

// these are the definitions needed for the routing table

class Packet;

// packet queue
struct PacketEntry {
  Packet *p;
  int data;
  struct PacketEntry *next;
};

// route entry
struct RouteEntry {
  Vector<unsigned> hops;
  unsigned hop_count;
  int lifetime_jiffies;
  int last_response_jiffies;
  bool source_route;
  struct RouteEntry *next;
};

struct RouteInfo {
  // maintanance data
  bool areq;
  bool polling;
  bool salvage;
  short requests;
  int poll_lifetime_jiffies;
  int last_poll_jiffies;
  int ttl;
  // route data
  unsigned dst;
  unsigned dst_seq;
  unsigned interface;
  unsigned last_hop_count;
  unsigned last_source_hop_count;
  unsigned char ethernet[6];
  HashMap<String, int> precursor_map;
  struct RouteEntry *shortest_route;
  struct RouteEntry *shortest_source_route;
  struct RouteEntry *rlist_head;
  // packet queue
  Spinlock *lock;
  PacketEntry *plist_head;
  PacketEntry *plist_tail;
  int p_entries;
};

// bytes 1 - 7
#define ADHOC_ETHERNET_ANNO 1

// byte 8
#define ADHOC_OFFSET_ANNO(p)	     ((p)->user_anno_c(8))
#define SET_ADHOC_OFFSET_ANNO(p, v)  ((p)->set_user_anno_c(8, (v)))

// byte 9
#define ADHOC_HEADER_ANNO(p)	     ((p)->user_anno_c(9))
#define SET_ADHOC_HEADER_ANNO(p, v)  ((p)->set_user_anno_c(9, (v)))

// byte 10
#define ADHOC_ID_ANNO(p)	     ((p)->user_anno_c(10))
#define SET_ADHOC_ID_ANNO(p, v)      ((p)->set_user_anno_c(10, (v)))


static const int RT_ROUTE_LIFETIME = 5000;
static const int RT_SOURCE_ROUTE_LIFETIME = -1;

static const int DELETE_PERIOD = 10000;
static const int RREP_WAIT_TIME = 10000;

static const unsigned RT_INFINITY = 0xffffff;
static const unsigned RT_UNKNOWN_IF = 255;

#ifdef CLICK_SIM
}
#endif

#endif
