#ifndef CLICK_DSR_H
#define CLICK_DSR_H

#include <netinet/in.h>

struct click_dsr {
  unsigned char dsr_next_header;
  unsigned char dsr_reserved;
  unsigned short dsr_len;
};

struct click_dsr_option {
  unsigned char dsr_type;
  unsigned char dsr_len;
};

struct click_dsr_rreq {
  unsigned char dsr_type;
  unsigned char dsr_len;
  unsigned short dsr_id;
  struct in_addr dsr_dst;
};

struct click_dsr_rrep {
  unsigned char dsr_type;
  unsigned char dsr_len;
  unsigned char dsr_flags;
  unsigned char dsr_pad; // used to handle the padding to a multiple of 4 bytes.
  unsigned short dsr_id;
};

struct click_dsr_rerr {
  unsigned char dsr_type;
  unsigned char dsr_len;
  unsigned char dsr_error;
  unsigned char dsr_flags;
  struct in_addr dsr_src;
  struct in_addr dsr_dst;
};

struct click_dsr_source {
  unsigned char dsr_type;
  unsigned char dsr_len;
  unsigned short dsr_flags;
  bool flag;
};

struct click_dsr_areq {
  unsigned char dsr_type;
  unsigned char dsr_len;
  unsigned short dsr_id;
  struct in_addr dsr_src;
};

struct click_dsr_ack {
  unsigned char dsr_type;
  unsigned char dsr_len;
  unsigned short dsr_id;
  struct in_addr dsr_src;
  struct in_addr dsr_dst;
};

struct click_dsr_addr {
  struct in_addr dsr_dst;
};

/* Constants used - need to check their values */
static const int DSR_RREQ_RETRIES = 2;
static const int DSR_NODE_TRAVERSAL_MS = 40;
static const int DSR_NET_DIAMETER = 35;
static const int DSR_NET_TRAVERSAL_MS = 3 * DSR_NODE_TRAVERSAL_MS * DSR_NET_DIAMETER / 2;
static const int DSR_RREQ_START_TTL = 2;
static const int DSR_RREQ_INCREMENT_TTL = 2;
static const int DSR_RREQ_THRESHOLD_TTL = 7;
static const int DSR_BROADCAST_RECORD_MS = 2 * DSR_NET_TRAVERSAL_MS;

#define DSR_TYPE_RREQ         2
#define DSR_TYPE_RREP         3
#define DSR_TYPE_RERR         4
#define DSR_TYPE_AREQ         5
#define DSR_TYPE_ACK          6
#define DSR_TYPE_SOURCE_ROUTE 7

#define IP_PROTO_DSR          200

#define DSR_NODE_UNREACHABLE  1
#define DSR_SALVAGE_LIMIT     4

#endif
