#include <stdio.h>
#include <string.h>
#include <click/list.h>
#include <click/path.h>

#define LC_MAX_NODES 50
// Specifies whether or not to retain links after they go bad
static bool lc_cache_negative_links = false;

// When inserting/deleting link A->B, also insert/delete B->A.
static bool lc_bidirectional_links = true;

// Evict "unreachable" links from the cache.
static bool lc_evict_unreachable_links = false;

// The Maximum no of nodes in a source route...
#define MAX_SR_LEN  15

///////////////////////////////////////////////////////////////////////////

class Link {
public:

//Constructor..Init the attributes..
	Link(int dst) {
		ln_dst = dst;
		ln_flags  = 0;
                ln_cost=0;
		ln_timeout = 0.0;
	//	ln_t = 0.0;
	}

	LIST_ENTRY(Link) ln_link;
	
//Link Attributes...


        int   ln_dst;
	int        ln_flags;       // link status information
#define LINK_FLAG_UP 0x01
	double        ln_cost;        // cost of using the link
	double     ln_timeout;     // when the link expires
#define LINK_TIMEOUT 10.0

	//Time	   ln_t;
};

LIST_HEAD(LinkHd, Link);

class LinkCache{

public:
	

        LinkCache(int);

//	void noticeDeadLink(const ID&from, const ID& to, Time t);
	// the link from->to isn't working anymore, purge routes containing
	// it from the cache

//	void noticeRouteUsed(const Path& route, Time t,
//			     const ID& who_from);
	// tell the cache about a route we saw being used

	void addRoute( Path route);
	// add this route to the cache (presumably we did a route request
	// to find this route and don't want to lose it)

	bool findRoute(int dest, Path& route, int for_me = 0);
	// if there is a cached path from us to dest returns true and fills in
	// the route accordingly. returns false otherwise
	
       int getNodeId() const;

//protected:
	LinkHd lcache[LC_MAX_NODES + 1];
	// note the zeroth index is not used

        // Add a link from->to, default cost=1

	int addLink(int  from, int  to,
		    int flags, double cost = 1, double timeout = LINK_TIMEOUT);


	//int addLink(int  from, int  to,
	//	    int flags, double cost, double timeout = LINK_TIMEOUT);
        // delete the link from ->to
	int delLink(int  from, int  to);



       // Look up the LinkCache to return a link from from->to
	Link* findLink(int from, int to);

       // Empty the linkcache
	void purgeLink(void);


       // Print out all the links of the linkcache..
	void dumpLink(void);

       // Print  out the minimum cost spanning tree..
	void dump_dijkstra(int dst);


	void dijkstra(void);
	//void Link_to_ID(Link* l, ID& id);

private:
	////////////////////////////////////////////////////////////
	// Dijkstra's Algorithm
        // My Node id
        int my_node_id;

        //Flag to indicate whether dijkstra needs to run again..
        int dirty;

       //Cost array..
	float d[LC_MAX_NODES + 1];


	int pi[LC_MAX_NODES + 1];

	bool S[LC_MAX_NODES + 1];

#define INFI 0x7fffffff


	void init_single_source(int s);


	void relax(int u, int v, float w);


	int  extract_min_q(void);
};

///////////////////////////////////////////////////////////////////////////






















