#ifndef PATHPOWER_H
#define PATHPOWER_H
#include <click/element.hh>
#include <click/etheraddress.hh>
#include <click/ipaddress.hh>
#include <click/timer.hh>
#include <click/hashmap.hh>
#include <assert.h>
#include <stdio.h>
#include <string.h>
#include <math.h>

class PathPower
{
// A PathPower is a sequence of power values for a given  route
// Hence they will have a private member as len that will
//store length of the route.
// Use appendToPath function to add a  power value to the Path instance
// Use the length function to obtain the length of the route
// Access the ith hop of the route as Path[i]

  private:
   	int len;
   	int cur_index;
   	float *_power;
   
  public :
  
  	PathPower()
    	{
		_power = new float[15];
      		len=0;
      		cur_index=0;
    	}

   	void reset()
   	{
     		len=0;
   	}  

    	void appendToPath(float power)
    	{
      		assert(len<15);//MAX_SRC_LEN
      		_power[len]=power;
     		len++;
    	}

    	float & operator[] (int n) const
    	{
      		return _power[n];
     	}

	PathPower reverse_route()
	{
		PathPower p_rev;
		p_rev.appendToPath(_power[0]);
		for (int i=len-1;i>=1;i--)
		{
			p_rev.appendToPath(_power[i]);
		}
		return p_rev;
	}
  
    	int length() const
    	{
      		return len;
    	} 
	
	PathPower truncate(int index)
	{
		int i;
		PathPower pwr_trunc;
		for (i=0;i<index;i++)
			pwr_trunc.appendToPath(_power[i]);
		return pwr_trunc;
	}
	
	PathPower converttomw()
	{
		PathPower pwr;
		pwr.appendToPath(0);
		for (int i=1;i<=len-1;i++)
		{
			double mw = pow(10.0 ,(double)(_power[i] /10.0));
			pwr.appendToPath((float)mw);
		}
		return  pwr;
	}
	PathPower converttodb()
	{
		PathPower pwr;
		pwr.appendToPath(0);
		for (int i=1;i<=len-1;i++)
		{
			double dbm = 10 * log10((double)_power[i]);
			pwr.appendToPath(dbm);
		}
		return  pwr;
	}
	
};
#endif
