#include <click/linkcache.hh>

LinkCache::LinkCache(int id) 
{
	
        int i;
        my_node_id=id;
	for(i = 0; i <= LC_MAX_NODES; i++) {
		LIST_INIT(&lcache[i]);
	}
	dirty = 1;
      

}



void
LinkCache::addRoute(Path rt)
{
  int c;
//Add each link of the path..
  for(c = 0; c < rt.length() - 1; c++) {
	  if(addLink(rt[c].node_id, rt[c+1].node_id, \
	     LINK_FLAG_UP,rt[c+1].to_cost)) {
	  }
  }
}



bool
LinkCache::findRoute(int dest, Path& route, int for_me)
{
	int v;

	int rpath[MAX_SR_LEN];

	int roff = 0;

	dumpLink();

	// Compute all of the shortest paths...
	if(dirty) 
        {
		dijkstra();
        
                //puts("Coming till after djisktra");
		// remove all of the links for nodes that are unreachable
	
         	if(lc_evict_unreachable_links == true) 
                     {
			purgeLink();
                 //puts("Coming till after purgelink");
	       	}
	}

	 dump_dijkstra(dest);

	// Trace backwards to figure out the path to DEST

	for(v = dest+1; d[v] < INFI; v = pi[v]) 
        {

		assert(v >= 1 && v <= LC_MAX_NODES);
		if (roff >= MAX_SR_LEN) {
 		        // path between us and dest is too long to be useful
		        break;
		}

		rpath[roff] = v-1;
		roff++;
                //I 've reached myself..thats the route from me to dest.
		if(v-1 == my_node_id)
			break;
	}

	if(roff < 2 || d[v] == INFI || roff >= MAX_SR_LEN) {
           printf("No route found from %d from %d\n",dest,my_node_id);
		return false; // no path
	}

	// Build the path that we need...
	ID id;

	route.reset();

	id.node_id = my_node_id;  // source route is rooted at "us"

	id.to_cost=0;
 
	route.appendToPath(id);

           //puts("Coming till after Append");

        //Now add the rest of the route..
	for(v = 1; v < roff ; v++) 
        {
		assert((int) (roff - v - 1) >= 0 && (roff - v - 1) < MAX_SR_LEN);

		Link *l = findLink(route[v-1].node_id, rpath[roff - v - 1]);
		


             //puts("Coming till after FindLink");
               assert(l);
               //Fill up the details of the hop
		id.node_id = rpath[roff - v - 1];
                id.to_cost= l->ln_cost;
	
               //Now add this hop to the route	
                route.appendToPath(id);

             //puts("Coming till after appendtopath");
	}
	
             //puts("Coming till end of fn");
	return true;
}


/*void
LinkCache::Link_to_ID(Link *l, ID& id)
{
	id.addr = l->ln_dst;
	id.type = ::IP;
	id.t = l->ln_t;
	id.link_type = l->ln_linktype;
	id.log_stat = l->ln_logstat;
}*/

///////////////////////////////////////////////////////////////////////////

/*
 * Dijkstra's algorithm taken from CLR.
 */

void
LinkCache::init_single_source(int s)
{
	int v;

	for(v = 1; v <= LC_MAX_NODES; v++) {
		d[v] = INFI;
		pi[v] = 0; // invalid node ID

		S[v] = false;
	}
	d[s+1] = 0;
}

void
LinkCache::relax(int u, int v, float w)
{
	assert(d[u] < INFI);

	if(d[v+1] > d[u] + w) {
		d[v+1] = d[u] + w;
		pi[v+1] = u;
	}
}

int
LinkCache::extract_min_q()
{
	int u;
	int min_u = 0;

	for(u = 1; u <= LC_MAX_NODES; u++) {
		if(S[u] == false && d[u] < INFI && (min_u == 0 || d[u] < d[min_u])) {
			min_u = u;
		}
	}
	return min_u; // (min_u == 0) ==> no valid link
}

void
LinkCache::dijkstra()
{
	int u;	
	Link *v;

	init_single_source(my_node_id);

	while((u = extract_min_q()) != 0) {
		S[u] = true;
		v = lcache[u].lh_first;
		for( ; v; v = v->ln_link.le_next) {
			if(v->ln_flags & LINK_FLAG_UP) {
				relax(u, v->ln_dst, v->ln_cost);
			}
		}
	}
	dirty = 0;  // all of the info is up-to-date
}

void
LinkCache::dump_dijkstra(int dst)
{
	static char tbuf[512];
	int u, toff = 0;

	bzero(tbuf, sizeof(tbuf));
	//sprintf(tbuf, "SRC  %d  dijkstra *%d* ",
	//	  node_id, dst);
	printf("SRC  %d  dijkstra *%d* ",
		  my_node_id, dst);
	toff = strlen(tbuf);

	for(u = 1; u <= LC_MAX_NODES; u++) {
		if(d[u] < INFI) {
			//sprintf(tbuf+toff, "%d,%d,%d ", u-1, d[u], pi[u]);
			printf("%d,%f,%d ", u-1, d[u], pi[u]-1);

//Important!!!!!! 
//node id's are stored from base 1 onwards..so pi array will return the base 1 address of node _id
// so to get the absolute address, we need to subtract 1 from it..
			//toff = strlen(tbuf);
			//assert(toff < sizeof(tbuf));
		}
	}

	//printf("%s\n", tbuf);
	//trace("%s", tbuf);
}
	

///////////////////////////////////////////////////////////////////////////

int
LinkCache::addLink(int from, int to,
		   int flags, double cost, double timeout)
{
	Link *l;
	int rc = 0;

	if((l = findLink(from, to)) == 0) {
		l = new Link(to);
		assert(l);
		LIST_INSERT_HEAD(&lcache[from+1], l, ln_link);
		dirty = 1;
		rc = 1;
	}
	l->ln_flags |= flags;
	l->ln_cost = cost;
	l->ln_timeout = timeout;

	if(lc_bidirectional_links == false)
		return rc;

	/*
	 * Add the "other" direction of the link...
	 */
	if((l = findLink(to, from)) == 0) {
		l = new Link(from);
		assert(l);
		LIST_INSERT_HEAD(&lcache[to+1], l, ln_link);
		dirty = 1;
	}
	l->ln_flags |= flags;
	l->ln_cost = cost;
	l->ln_timeout = timeout;

	return rc;
}

int
LinkCache::delLink(int from, int to)
{
	Link *l;
	int rc = 0;

	if((l = findLink(from, to))) {
		LIST_REMOVE(l, ln_link);
		delete l;
		dirty = 1;
		rc = 1;
	}

	if(lc_bidirectional_links == false)
		return rc;

	/*
	 * Remove the "other" direction of the link...
	 */
	if((l = findLink(to, from))) {
		LIST_REMOVE(l, ln_link);
		delete l;
		dirty = 1;
	}

	return rc;
}

Link*
LinkCache::findLink(int from, int to)
{
	Link *l;

	for(l = lcache[from+1].lh_first; l; l = l->ln_link.le_next) {
		if(l->ln_dst == to) {
			return l;
		}
	}
	return 0;
}

void
LinkCache::purgeLink()
{
	int u;
	Link *l;

	for(u = 1; u <= LC_MAX_NODES; u++) {
		if(d[u] == INFI) {
			int from, to;
			from= u-1;

			l = lcache[u].lh_first;
			for( ; l; l = l->ln_link.le_next) {
				(void) delLink(from, to);
			}
		}
	}
}


void
LinkCache::dumpLink()
{
	Link *l;
	int i;
	static char tbuf[512];
	int toff = 0;
	bzero(tbuf, sizeof(tbuf));
	//sprintf(tbuf, "SRC  _%s_ dump-link ",  node_id);
	printf("SRC  _%d_ dump-link ",  my_node_id);
	toff = strlen(tbuf);

	for(i = 1; i <= LC_MAX_NODES; i++) {
		for(l = lcache[i].lh_first; l; l = l->ln_link.le_next) {
			//sprintf(tbuf+toff, "%d->%d, ", i-1, l->ln_dst);
			printf("%d->%d ", i-1, l->ln_dst);
		}
	}

}

int LinkCache::getNodeId() const
{
	return my_node_id;
}
